#include "__Lib_nRF24L01.h"
#include "nRF24L01_defs.h"
#include "led_protocol.h"

/*
 * Project name:
     nRFControlSlave
 * Test configuration:
     MCU:             LM3S9B95
     Dev.Board:       EasyMX PRO v7 for Stellaris ARM
                      http://www.mikroe.com/easymx-pro/stellaris/
     Oscillator:      72.0 MHz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/

     Note: Turn off all leds except leds on port D.
*/

/*******************************************************************************
* nRF24L01+ connections
*******************************************************************************/
sbit nRF_CS at GPIO_PORTG_DATA0_bit;
sbit nRF_CE at GPIO_PORTE_DATA2_bit;
sbit nRF_IRQ at GPIO_PORTH_DATA0_bit;
sbit nRF_CS_Direction at GPIO_PORTG_DIR0_bit;
sbit nRF_CE_Direction at GPIO_PORTE_DIR2_bit;
sbit nRF_IRQ_Direction at GPIO_PORTH_DIR0_bit;

TnRF_Status nRFStatus;
TRF_Config rfConfig;

unsigned char rfState;

TLed leds[8];

//Initialization of Leds data
void InitLeds(void) {
  unsigned char i;
  
  for (i = 0; i < 8; i++)
  {
    leds[i]._state = 0;
    leds[i]._no    = i;
  }
}

//Initialization of Timer 2
void InitTimer2(){
  SYSCTL_RCGC1_TIMER2_bit = 1;
  EnableInterrupts();
  TIMER_CTL_TAEN_TIMER2_CTL_bit = 0;
  TIMER2_CFG   = 4;
  TIMER2_TAMR |= 2;
  TIMER2_TAPR  = 0;
  TIMER2_TAILR = 1280;
  NVIC_IntEnable(IVT_INT_TIMER2A);
  TIMER_IMR_TATOIM_TIMER2_IMR_bit = 1;
}

//Initialization of External Interrupt
void InitExt(){
  GPIO_PORTH_AFSEL = _GPIO_PINMASK_0;   // Enable alternate pin function
  GPIO_PORTH_IS    = 0;        // Edge sensitive
  GPIO_PORTH_IBE   = 0;
  GPIO_PORTH_IEV   = 0;        // Falling edge will trigger the event
  GPIO_PORTH_IM    = _GPIO_PINMASK_0;   // Set mask to PG7
  NVIC_IntEnable(IVT_INT_GPIOH);
}

//Interrupt function for Timer 2
void Timer2A_interrupt() iv IVT_INT_TIMER2A ics ICS_AUTO {
  TIMER_ICR_TATOCINT_TIMER2_ICR_bit = 1;
  TIMER2_IRQ();
  TIMER_CTL_TAEN_TIMER2_CTL_bit = 0;
}

//Interrupt function for External interrupt
void GPIOInt() iv IVT_INT_GPIOH ics ICS_AUTO {
  GPIO_PORTH_ICR = _GPIO_PINMASK_0;

  nRF24L01_IRQ();
}

void InitRF()
{
  nRF_CE = 0;
  nRF_CS = 1;

  //Initialize timer and external interrupt
  InitTimer2();
  InitExt();

  //Initialize SPI over A port
  SPI0_Init_Advanced(2000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI0_A245);
}

void SetVal(unsigned char off, unsigned char *val)
{
  if(off == LED_OFF)
  {
    *val = 0;
  }
  else
  {
    *val = 1;
  }
}

//Command used to control all leds on port (On/Off state)
void CmdAll(unsigned char off)
{
  unsigned char val, i;
  val = 0;
  SetVal(off, &val);
  
  for(i = 0; i < 8; i++)
  {
    leds[i]._state = val;
    
    if(val == LED_ON)
    {
      GPIO_PORTD_DATA |= val << i;
    }
    else
    {
      GPIO_PORTD_DATA &= val << i;
    }
  }
}

//Command used to control leds in a column.
void CmdColumn(unsigned char off)
{
  unsigned char val, i;
  val = 0;
  SetVal(off, &val);

  if((RX_pload[0] & 0x0F) == LED_CLN_01)
  {
    if(val == 1)
      GPIO_PORTD_DATA |= (val << 4) | (val << 5) | (val << 6) | (val << 7);
    else
      GPIO_PORTD_DATA &= 0x0F;
      
    for(i = 4; i < 8; i++)
    {
      leds[i]._state = val;
    }
  }
  else
  {
    if(val == 1)
      GPIO_PORTD_DATA |= val | (val << 1) | (val << 2) | (val << 3);
    else
      GPIO_PORTD_DATA &= 0xF0;
      
    for(i = 0; i < 4; i++)
    {
      leds[i]._state = val;
    }
  }
}

//Command used to control all leds in a row.
void CmdRow(unsigned char off)
{
  unsigned char val, l1, l2, tmp;;
  val = 0;
  
  SetVal(off, &val);
  
  l1 = 7 - (RX_pload[0] & 0x0F);
  l2 = 3 - (RX_pload[0] & 0x0F);
  
  leds[l1]._state = val;
  leds[l2]._state = val;
  
  tmp = (1 << l1) | (1 << l2);
  
  if(val == 1)
    GPIO_PORTD_DATA |= (val << l1) | (val << l2);
  else
    GPIO_PORTD_DATA &= ~tmp;
}

//Command used to control a single Led.
void CmdSingleLed(unsigned char off)
{
  unsigned char val, tmp;
  
  val = 0;
  SetVal(off, &val);
  
  leds[((RX_pload[0] & 0x0F) - 1)]._state = val;
  
  tmp = (1 << ((RX_pload[0] & 0x0F) - 1));
  
  if(val == 0)
    GPIO_PORTD_DATA &= ~tmp;
  else
    GPIO_PORTD_DATA |= tmp;
}

void ProcessCmd(unsigned char *cmd)
{
  //Get command bits from RX buffer.
  switch((RX_pload[0] & 0xF0))
  {
    case LED_CMD_ALL_ON :
    {
      CmdAll(LED_ON); break;
    }
    
    case LED_CMD_SINGLE_ON:
    {
      CmdSingleLed(LED_ON);
      break;
    }
    
    case LED_CMD_ROW_ON:
    {
      CmdRow(LED_ON);
      break;
    }
    
    case LED_CMD_COLUMN_ON:
    {
      CmdColumn(LED_ON);
      break;
    }
    
    case LED_CMD_ALL_OFF:
    {
      CmdAll(LED_OFF); break;
    }
    
    case LED_CMD_SINGLE_OFF:
    {
      CmdSingleLed(LED_OFF);
      break;
    }
    
    case LED_CMD_ROW_OFF:
    {
      CmdRow(LED_OFF);
      break;
    }
    
    case LED_CMD_COLUMN_OFF:
    {
      CmdColumn(LED_OFF);
      break;
    }
  }
}

void main() {
  GPIO_Digital_Output(&GPIO_PORTD_DATA_BITS, _GPIO_PINMASK_ALL);
  GPIO_PORTD_DATA = 0;

  InitLeds();

  //nRF Config Settings
  rfConfig.autoTransmitDelay = nRF_AUTO_RETRANSMIT_DELAY_4000us; //250 + 15 * 250 = 4000us
  rfConfig.autoTransmitCount = nRF_AUTO_RETRANSMIT_COUNT_05;
  rfConfig.channel           = 64;
  rfConfig.outputPower       = nRF_OUTPUT_POWER_0dBm;
  rfConfig.dataRate          = nRF_DATA_RATE_1Mbps;

  nRF_Init();
  InitRF();
  Delay_ms(100);

  rfState = RF_STOP;
  
  nRF_RX_Mode_Init_Advanced(rfConfig.dataRate, rfConfig.outputPower, rfConfig.channel);

  while (1) {
    if(nRF_Receive_Packet())
    {
      nRF_Process(&nRFStatus);
      if(nRFStatus._status == nRF_RX_DR)
      {
          ProcessCmd(RX_pload);
      }
    }
  }


}